import * as React from 'react';
import PropTypes from 'prop-types';
import DropdownMenu, { DropdownMenuProps, UseDropdownMenuMetadata, UseDropdownMenuOptions } from './DropdownMenu';
import DropdownToggle, { DropdownToggleProps, UseDropdownToggleMetadata } from './DropdownToggle';
import { DropdownItemProps } from './DropdownItem';
import { SelectCallback } from './types';
import { Placement } from './usePopper';
export type { DropdownMenuProps, UseDropdownMenuMetadata, UseDropdownMenuOptions, DropdownToggleProps, UseDropdownToggleMetadata, DropdownItemProps, };
export interface DropdownInjectedProps {
    onKeyDown: React.KeyboardEventHandler;
}
export interface ToggleMetadata {
    source: string | undefined;
    originalEvent: React.SyntheticEvent | Event | undefined;
}
export interface DropdownProps {
    placement?: Placement;
    defaultShow?: boolean;
    show?: boolean;
    onSelect?: SelectCallback;
    onToggle?: (nextShow: boolean, meta: ToggleMetadata) => void;
    itemSelector?: string;
    focusFirstItemOnShow?: false | true | 'keyboard';
    children: React.ReactNode;
}
/**
 * @displayName Dropdown
 * @public
 */
declare function Dropdown({ defaultShow, show: rawShow, onSelect, onToggle: rawOnToggle, itemSelector, focusFirstItemOnShow, placement, children, }: DropdownProps): JSX.Element;
declare namespace Dropdown {
    var displayName: string;
    var propTypes: {
        /**
         * A render prop that returns the root dropdown element. The `props`
         * argument should spread through to an element containing _both_ the
         * menu and toggle in order to handle keyboard events for focus management.
         *
         * @type {Function ({
         *   props: {
         *     onKeyDown: (SyntheticEvent) => void,
         *   },
         * }) => React.Element}
         */
        children: PropTypes.Requireable<PropTypes.ReactNodeLike>;
        /**
         * The PopperJS placement for positioning the Dropdown menu in relation to it's Toggle.
         *
         * @default 'bottom-start'
         */
        placement: PropTypes.Requireable<import("@popperjs/core").Placement>;
        /**
         * Controls the focus behavior for when the Dropdown is opened. Set to
         * `true` to always focus the first menu item, `keyboard` to focus only when
         * navigating via the keyboard, or `false` to disable completely
         *
         * The Default behavior is `false` **unless** the Menu has a `role="menu"`
         * where it will default to `keyboard` to match the recommended [ARIA Authoring practices](https://www.w3.org/TR/wai-aria-practices-1.1/#menubutton).
         */
        focusFirstItemOnShow: PropTypes.Requireable<string | boolean>;
        /**
         * A css slector string that will return __focusable__ menu items.
         * Selectors should be relative to the menu component:
         * e.g. ` > li:not('.disabled')`
         */
        itemSelector: PropTypes.Requireable<string>;
        /**
         * Whether or not the Dropdown is visible.
         *
         * @controllable onToggle
         */
        show: PropTypes.Requireable<boolean>;
        /**
         * Sets the initial show position of the Dropdown.
         */
        defaultShow: PropTypes.Requireable<boolean>;
        /**
         * A callback fired when the Dropdown wishes to change visibility. Called with the requested
         * `show` value, the DOM event, and the source that fired it: `'click'`,`'keydown'`,`'rootClose'`, or `'select'`.
         *
         * ```ts static
         * function(
         *   isOpen: boolean,
         *   event: SyntheticEvent,
         * ): void
         * ```
         *
         * @controllable show
         */
        onToggle: PropTypes.Requireable<(...args: any[]) => any>;
    };
    var Menu: typeof DropdownMenu;
    var Toggle: typeof DropdownToggle;
    var Item: import("./types").DynamicRefForwardingComponent<import("./types").DynamicRefForwardingComponent<"button", import("./Button").ButtonProps>, DropdownItemProps>;
}
export default Dropdown;
